# Copyright 2021-2022 by TRSTN4. All rights reserved.
# This file is part of the br4nch python package, and is released under the "GNU General Public License v3.0".
# Please see the LICENSE file that should have been included as part of this package.

"""
Dictionaries:
  - branches: Stores the given header and layers in this dictionary for each branch.
  - output: Stores the final result of the 'builder' class in this dictionary for each branch.
  - uids: Stores all generated UIDS in this dictionary for each branch.
  - sizes: Stores all given sizes in this dictionary for each branch.
  - symbols: Stores all given 'line', 'split' and 'end' strings in this dictionary for each branch.
  - paint_branch: Stores all given paint for the branch in this dictionary for each branch.
  - paint_header: Stores all given header paint in this dictionary for each branch.
  - paint_layer: Stores all given paint for the layers in this dictionary for each branch.
"""
branches = {}
output = {}
uids = {}
sizes = {}
symbols = {}
paint_branch = {}
paint_header = {}
paint_layer = {}
